@@ -16,6 +16,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import threading
 from .util import UnicodeMixin, ImmutableMixin, mutating_method
 from .util import u, unicod, rpr, force_unicode
 
@@ -228,6 +229,14 @@ class PhoneNumberDesc(UnicodeMixin, ImmutableMixin):
         return result
 
 
+def _same_pattern(left, right):
+    if left is None and right is None:
+        return True
+    if left is None or right is None:
+        return False
+    return (left.national_number_pattern == right.national_number_pattern)
+
+
 class PhoneMetadata(UnicodeMixin, ImmutableMixin):
     """Class representing metadata for international telephone numbers for a region.
 
@@ -241,6 +250,10 @@ class PhoneMetadata(UnicodeMixin, ImmutableMixin):
     to warn about breaking changes.
 
     """
+    # Lock that protects the *_available fields while they are being modified.
+    # The modificiation involves loading data from a file, so we cannot just
+    # rely on the GIL.
+    _metadata_lock = threading.Lock()
     # If a region code is a key in this dict, metadata for that region is available.
     # The corresponding value of the map is either:
     #   - a function which loads the region's metadata
@@ -264,8 +277,10 @@ class PhoneMetadata(UnicodeMixin, ImmutableMixin):
         loader = kls._region_available.get(region_code, None)
         if loader is not None:
             # Region metadata is available but has not yet been loaded.  Do so now.
+            kls._metadata_lock.acquire()
             loader(region_code)
             kls._region_available[region_code] = None
+            kls._metadata_lock.release()
         return kls._region_metadata.get(region_code, default)
 
     @classmethod
@@ -273,8 +288,10 @@ class PhoneMetadata(UnicodeMixin, ImmutableMixin):
         loader = kls._short_region_available.get(region_code, None)
         if loader is not None:
             # Region short number metadata is available but has not yet been loaded.  Do so now.
+            kls._metadata_lock.acquire()
             loader(region_code)
             kls._short_region_available[region_code] = None
+            kls._metadata_lock.release()
         return kls._short_region_metadata.get(region_code, default)
 
     @classmethod
@@ -282,8 +299,10 @@ class PhoneMetadata(UnicodeMixin, ImmutableMixin):
         loader = kls._country_code_available.get(country_code, None)
         if loader is not None:
             # Region metadata is available but has not yet been loaded.  Do so now.
+            kls._metadata_lock.acquire()
             loader(country_code)
             kls._country_code_available[country_code] = None
+            kls._metadata_lock.release()
         return kls._country_code_metadata.get(country_code, default)
 
     @classmethod
@@ -309,14 +328,14 @@ class PhoneMetadata(UnicodeMixin, ImmutableMixin):
     def load_all(kls):
         """Force immediate load of all metadata"""
         # Force expansion of contents to lists because we invalidate the iterator
-        for region_code, loader in list(kls._region_available.items()):
-            if loader is not None:  # pragma no cover
-                loader(region_code)
+        for region_code, region_loader in list(kls._region_available.items()):
+            if region_loader is not None:  # pragma no cover
+                region_loader(region_code)
                 kls._region_available[region_code] = None
-        for country_code, loader in list(kls._country_code_available.items()):
-            if loader is not None:
-                loader(country_code)
-                kls._country_code_available[region_code] = None
+        for country_code, cc_loader in list(kls._country_code_available.items()):
+            if cc_loader is not None:
+                cc_loader(country_code)
+                kls._country_code_available[country_code] = None
 
     @mutating_method
     def __init__(self,
@@ -336,6 +355,7 @@ class PhoneMetadata(UnicodeMixin, ImmutableMixin):
                  short_code=None,
                  standard_rate=None,
                  carrier_specific=None,
+                 sms_services=None,
                  no_international_dialling=None,
                  country_code=None,
                  international_prefix=None,
@@ -374,6 +394,7 @@ class PhoneMetadata(UnicodeMixin, ImmutableMixin):
         self.short_code = short_code  # None or PhoneNumberDesc
         self.standard_rate = standard_rate  # None or PhoneNumberDesc
         self.carrier_specific = carrier_specific  # None or PhoneNumberDesc
+        self.sms_services = sms_services  # None or PhoneNumberDesc
 
         # The rules here distinguish the numbers that are only able to be
         # dialled nationally.
@@ -447,7 +468,7 @@ class PhoneMetadata(UnicodeMixin, ImmutableMixin):
         # not.  This is used to speed up determining phone number type in
         # countries where these two types of phone numbers can never be
         # distinguished.
-        self.same_mobile_and_fixed_line_pattern = (self.mobile == self.fixed_line)
+        self.same_mobile_and_fixed_line_pattern = _same_pattern(self.mobile, self.fixed_line)
 
         # Note that the number format here is used for formatting only, not
         # parsing.  Hence all the varied ways a user *may* write a number need
@@ -520,20 +541,26 @@ class PhoneMetadata(UnicodeMixin, ImmutableMixin):
         if register:
             # Register this instance with the relevant class-wide map
             if self.id == REGION_CODE_FOR_NON_GEO_ENTITY:
-                kls_map = PhoneMetadata._country_code_metadata
-                id = self.country_code
+                if self.country_code in PhoneMetadata._country_code_metadata:
+                    other = PhoneMetadata._country_code_metadata[self.country_code]
+                    if self != other:
+                        raise Exception("Duplicate non-geo PhoneMetadata for %s (from %s:%s)" % (self.country_code, self.id, self.country_code))
+                else:
+                    PhoneMetadata._country_code_metadata[self.country_code] = self
             elif self.short_data:
-                kls_map = PhoneMetadata._short_region_metadata
-                id = self.id
+                if self.id in PhoneMetadata._short_region_metadata:
+                    other = PhoneMetadata._short_region_metadata[self.id]
+                    if self != other:
+                        raise Exception("Duplicate short PhoneMetadata for %s (from %s:%s)" % (self.id, self.id, self.country_code))
+                else:
+                    PhoneMetadata._short_region_metadata[self.id] = self
             else:
-                kls_map = PhoneMetadata._region_metadata
-                id = self.id
-            if id in kls_map:
-                other = kls_map[id]
-                if self != other:
-                    raise Exception("Duplicate PhoneMetadata for %s (from %s:%s)" % (id, self.id, self.country_code))
-            else:
-                kls_map[id] = self
+                if self.id in PhoneMetadata._region_metadata:
+                    other = PhoneMetadata._region_metadata[self.id]
+                    if self != other:
+                        raise Exception("Duplicate PhoneMetadata for %s (from %s:%s)" % (self.id, self.id, self.country_code))
+                else:
+                    PhoneMetadata._region_metadata[self.id] = self
 
     def __eq__(self, other):
         if not isinstance(other, PhoneMetadata):
@@ -579,6 +606,8 @@ class PhoneMetadata(UnicodeMixin, ImmutableMixin):
             result += unicod(",\n    standard_rate=%s") % self.standard_rate
         if self.carrier_specific is not None:
             result += unicod(",\n    carrier_specific=%s") % self.carrier_specific
+        if self.sms_services is not None:
+            result += unicod(",\n    sms_services=%s") % self.sms_services
         if self.no_international_dialling is not None:
             result += unicod(",\n    no_international_dialling=%s") % self.no_international_dialling
 
